#include <xc.h>
#include "configbits-16ex8.h"
#include "SDlib16.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <proc/p32mx695f512h.h>
#define T1flag  IFS0bits.T1IF
#define DACselect  LATGbits.LATG9
#define ClipThresh 0.64
#define ReverbSize 1000
#define SIZE        40


int ADCinit(void);
void serial_init1(unsigned long);
int enable_SPI2(void);
int SPI2out(int);
int T1_init(void);
void UARTinit(void);
int Clipping(short);


int main(void) {
    ADCinit();
    int sample = 0;
    TRISE = 0;
    LATE = 0xFF;
    T1_init();
    enable_SPI2();
    TRISBbits.TRISB8=0;
    LATBbits.LATB8=0;
    int junk;
    int doOverdrive = 0;
    UARTinit();  
    OSCCONbits.PBDIV = 0b00;
    LATE = 0xFF;
    while(1) {
        
        if(U1STAbits.URXDA) {
            junk = U1RXREG;
            U1STAbits.URXDA = 0;
            doOverdrive = !doOverdrive;
        }

        if(T1flag) {
            T1flag = 0;
            sample = ADConvert();
            if(doOverdrive) {
                sample = Clipping(sample);
                LATE = 0x00;
            }
            else {
                LATE = 0xFF;
            }
            sendToDAC(0);
            sendToDAC(sample);
        }
    }


    return 0;
}

void UARTinit(void){
    serial_init1(9600);
    U1STAbits.URXDA = 0;
}

int ADCinit(void) {

    TRISBbits.TRISB4  = 1;
    AD1PCFGbits.PCFG4 = 0;

    AD1CON1bits.FORM = 0;

    AD1CON1bits.SSRC = 0;
    AD1CON2bits.VCFG = 0;
    AD1CON2bits.CSCNA = 0;
    AD1CON2bits.BUFM = 0;
    AD1CON2bits.ALTS = 0;

    AD1CON3bits.ADRC = 0;

    AD1CON3bits.ADCS = 7;
    AD1CON1bits.ON = 1;

    return 0;
}

int ADConvert(void) {
    int i=0;
    int stuff = 0;

    AD1CHSbits.CH0SA = 4;
    AD1CON1bits.SAMP = 1;
    for(i=0;i<100;i++);
    AD1CON1bits.SAMP = 0;
    while( !AD1CON1bits.DONE );

    stuff = ADC1BUF0;

    return stuff;

}

int T1_init(void) {
    OSCCONbits.PBDIV = 0b00;
    TMR1 = 0;
    T1CONbits.TCS = 0;
    T1CONbits.TCKPS = 0b00;
    PR1 = 2000;
    T1flag = 0;
    T1CONbits.ON = 1;
}

int enable_SPI2(void){
    //SPI2 controls the DAC
    SPI2CONbits.ON  = 0;
    SPI2CONbits.CKE = 0;
    SPI2CONbits.CKP = 1;
    SPI2CONbits.SMP = 1;
    SPI2CONbits.MSTEN = 1;
    SPI2BRG = 2;            //80MHz div 5 = 16MHz, 20MHz max
    SPI2STATbits.SPIROV = 0;
    SPI2CONbits.MODE32 = 0;
    SPI2CONbits.MODE16 = 0;
    SPI2CONbits.SSEN = 0;

    TRISGbits.TRISG8 = 0;
    TRISGbits.TRISG6 = 0;
    TRISGbits.TRISG9 = 0;
    DACselect = 1;

    SPI2CONbits.ON = 1;


    return 0;
}

int SPI2out(int byte) {
    int temp = 0;
    while(!SPI2STATbits.SPITBE);
    SPI2BUF = byte;
    while(!SPI2STATbits.SPIRBF);
    temp = SPI2BUF;
    return temp;
}

int sendToDAC(int sample) {
    sample = sample<<2;             //10 bits --> 12 bits
    int command = 0b00110000;
    int MSB = (sample>>8) + command; //command followed by top 4 bits
    int LSB = sample & 0x00FF;       //bottom 8 bits
    DACselect = 0;
    SPI2out(MSB);
    SPI2out(LSB);
    DACselect = 1;


}

int Clipping(short input) {
    static int array[SIZE];
    static int i=0;
    int j;
    static short maximum = 0;
    short output;
    
    array[i] = input;
    i++;
        
    if(i>=SIZE) {
        i = 0;
    }
    
    for (j = 0; j<=i; j++) {
        if(array[j] > maximum) {
            maximum = array[j];
        }
    }
    

        if(input >= ClipThresh*maximum) {
            output = ClipThresh*maximum;
        }
        else if(input <= -ClipThresh*maximum) {
            output = -ClipThresh*maximum;
        }
        else {
            output = input;
        }



    
    return output;
}

void serial_init1(unsigned long rate) {
    unsigned long pbclock;
    
    U1MODEbits.ON = 0;
    U1MODEbits.BRGH = 0;
    U1STAbits.UTXEN = 1;
    U1STAbits.URXEN = 1;
    pbclock = get_pb_clock();
    U1BRG = (pbclock/(16*rate)) - 1;
    U1MODEbits.PDSEL = 0b00;
    U1MODEbits.STSEL = 0; 
    U1MODEbits.ON = 1;
    
    return;
}